% Creates 10 folds by applying 10 fold cross-validation to each class
% (so in each fold, the test set contains 10% of data, the
% training/val set contains the remaining 90%, where val set (10%) is
% located directly below test set. Each fold contains a completely different test
% set, meaning a single point can never occur twice in all the test sets).
% the percentages shown above are valid for the two classes (stratified
% sampling).
% Note that we do NOT impose artificial imbalance in this file.

%% load the bookcrossing data (similar for other data sources)
try
    addpath(genpath('C:\LIBLINEAR'))%location of LIBLINEAR toolbox
catch
    disp('change line 12 in code to location where you stored LIBLINEAR toolbox')
end
[Y,X] = libsvmread('.\BookCrossing\bookcrossing_bin.libsvm');

%% process the data  (for other data sources, check the C1 and C2 assignments)
%remove columns with all zeros
[~,j,~] = find(X);
X = X(:,unique(j));
%remove rows with all zeros
[i,~,~] = find(X);
X = X(unique(i),:);
Y = Y(unique(i),:);
%randomize data
perm = randperm(size(X,1));
X = X(perm,:);
Y = Y(perm);
%split the data in 2 classes (take care when using other data sources)
C1_ind = find(Y == 0);Y(C1_ind) = -1;nr_C1 = length(C1_ind);%C1 
C2_ind = find(Y == 1);Y(C2_ind) = 1; nr_C2 = length(C2_ind);%C2

X_C1 = X(C1_ind,:);Y_C1 = Y(C1_ind);
X_C2 = X(C2_ind,:);Y_C2 = Y(C2_ind);

%sizes of data sets
test_size_C1 = floor(0.1*nr_C1);
train_size_C1 = ceil(0.8*nr_C1);
val_size_C1 = nr_C1 - test_size_C1 - train_size_C1;
test_size_C2 = floor(0.1*nr_C2);
train_size_C2 = ceil(0.8*nr_C2);
val_size_C2 = nr_C2 - test_size_C2 - train_size_C2;

%% create 10 seperate folds

Folds = cell(10,6);%init

for fold = 1:9
    %class C1
    test_ind_start_C1 = (fold-1)*test_size_C1+1;
    val_ind_start_C1 = test_ind_start_C1+test_size_C1;
    train_ind_start_C1 = val_ind_start_C1 + val_size_C1;
   
    X_C1_test = X_C1(test_ind_start_C1:test_ind_start_C1+test_size_C1-1,:);
    Y_C1_test = Y_C1(test_ind_start_C1:test_ind_start_C1+test_size_C1-1);
    X_C1_val = X_C1(val_ind_start_C1:val_ind_start_C1+val_size_C1-1,:);
    Y_C1_val = Y_C1(val_ind_start_C1:val_ind_start_C1+val_size_C1-1);
    X_C1_train = [X_C1(train_ind_start_C1:end,:);
                  X_C1(1:test_ind_start_C1-1,:)];
    Y_C1_train = [Y_C1(train_ind_start_C1:end);
                  Y_C1(1:test_ind_start_C1-1)];
   
    %class C2
    test_ind_start_C2 = (fold-1)*test_size_C2+1;
    val_ind_start_C2 = test_ind_start_C2+test_size_C2;
    train_ind_start_C2 = val_ind_start_C2 + val_size_C2;
    
    X_C2_test = X_C2(test_ind_start_C2:test_ind_start_C2+test_size_C2-1,:);
    Y_C2_test = Y_C2(test_ind_start_C2:test_ind_start_C2+test_size_C2-1);
    X_C2_val = X_C2(val_ind_start_C2:val_ind_start_C2+val_size_C2-1,:);
    Y_C2_val = Y_C2(val_ind_start_C2:val_ind_start_C2+val_size_C2-1);
    X_C2_train = [X_C2(train_ind_start_C2:end,:);
                  X_C2(1:test_ind_start_C2-1,:)];
    Y_C2_train = [Y_C2(train_ind_start_C2:end);
                  Y_C2(1:test_ind_start_C2-1)];
    
    %store current fold
    Folds{fold,1} = [X_C1_train;X_C2_train];
    Folds{fold,2} = [Y_C1_train;Y_C2_train];
    Folds{fold,3} = [X_C1_val;X_C2_val];
    Folds{fold,4} = [Y_C1_val;Y_C2_val];    
    Folds{fold,5} = [X_C1_test;X_C2_test];
    Folds{fold,6} = [Y_C1_test;Y_C2_test];
end
%fold 10 is 'special' since validation set also appears partly in front
fold = 10;
%class C1
test_ind_start_C1 = (fold-1)*test_size_C1+1;
X_C1_test = X_C1(test_ind_start_C1:test_ind_start_C1+test_size_C1-1,:);
Y_C1_test = Y_C1(test_ind_start_C1:test_ind_start_C1+test_size_C1-1);
val_ind_start_C1 = test_ind_start_C1+test_size_C1;
if (val_ind_start_C1 > nr_C1)
    val_ind_start_C1 = 1;
    train_ind_start_C1 = val_ind_start_C1 + val_size_C1;    
    X_C1_val = X_C1(val_ind_start_C1:train_ind_start_C1-1,:);
    Y_C1_val = Y_C1(val_ind_start_C1:train_ind_start_C1-1);
    X_C1_train = X_C1(train_ind_start_C1:test_ind_start_C1-1,:);
    Y_C1_train = Y_C1(train_ind_start_C1:test_ind_start_C1-1);
else
    nr_val_final_C1 = nr_C1-val_ind_start_C1+1;
    train_ind_start_C1 = val_size_C1-nr_val_final_C1+1;
    X_C1_val = [X_C1(val_ind_start_C1:end,:);
                X_C1(1:train_ind_start_C1-1,:)];
    Y_C1_val = [Y_C1(val_ind_start_C1:end);
                Y_C1(1:train_ind_start_C1-1)]; 
    X_C1_train = X_C1(train_ind_start_C1:test_ind_start_C1-1,:);
    Y_C1_train = Y_C1(train_ind_start_C1:test_ind_start_C1-1);    
end
%class C2
test_ind_start_C2 = (fold-1)*test_size_C2+1;
X_C2_test = X_C2(test_ind_start_C2:test_ind_start_C2+test_size_C2-1,:);
Y_C2_test = Y_C2(test_ind_start_C2:test_ind_start_C2+test_size_C2-1);
val_ind_start_C2 = test_ind_start_C2+test_size_C2;
if (val_ind_start_C2 > nr_C2)
    val_ind_start_C2 = 1;
    train_ind_start_C2 = val_ind_start_C2 + val_size_C2;    
    X_C2_val = X_C2(val_ind_start_C2:train_ind_start_C2-1,:);
    Y_C2_val = Y_C2(val_ind_start_C2:train_ind_start_C2-1);
    X_C2_train = X_C2(train_ind_start_C2:test_ind_start_C2-1,:);
    Y_C2_train = Y_C2(train_ind_start_C2:test_ind_start_C2-1);
else
    nr_val_final_C2 = nr_C2-val_ind_start_C2+1;
    train_ind_start_C2 = val_size_C2-nr_val_final_C2+1;
    X_C2_val = [X_C2(val_ind_start_C2:end,:);
                X_C2(1:train_ind_start_C2-1,:)];
    Y_C2_val = [Y_C2(val_ind_start_C2:end);
                Y_C2(1:train_ind_start_C2-1)]; 
    X_C2_train = X_C2(train_ind_start_C2:test_ind_start_C2-1,:);
    Y_C2_train = Y_C2(train_ind_start_C2:test_ind_start_C2-1);    
end
%store current fold
Folds{fold,1} = [X_C1_train;X_C2_train];
Folds{fold,2} = [Y_C1_train;Y_C2_train];
Folds{fold,3} = [X_C1_val;X_C2_val];
Folds{fold,4} = [Y_C1_val;Y_C2_val];    
Folds{fold,5} = [X_C1_test;X_C2_test];
Folds{fold,6} = [Y_C1_test;Y_C2_test];    

%save all the folds 
save('Example_Folds_orig_BookCrossing.mat','-v7.3','Folds')
    