function Z = Matrix_Multiplication_Blocks(X,Y,Block_Size_X,Block_Size_Y)
%MATRIX_MULTIPLICATION_BLOCKS multiplies two sparse matrices X and Y (nr of
%columns in X = nr of rows in Y) in blocks to avoid memory errors.
try
%init
Nr_Rows_X = size(X,1);
Nr_Cols_Y = size(Y,2);
Nr_Blocks_X = floor(Nr_Rows_X/Block_Size_X);
Nr_Blocks_Y = floor(Nr_Cols_Y/Block_Size_Y);
%[i,j,v] = find(Z), with Z = X*Y (the desired result)
i = zeros(nnz(X),1);j = zeros(nnz(X),1);v = zeros(nnz(X),1);
Pos_Current = 1;
%process X_blocks with full blocksize
for X_Block_Ind = 1:Nr_Blocks_X
    X_Block = X((X_Block_Ind-1)*Block_Size_X+1:X_Block_Ind*Block_Size_X,:);
    %process Y_block with full blocksize
    for Y_Block_Ind = 1:Nr_Blocks_Y
        Y_Block = Y(:,(Y_Block_Ind-1)*Block_Size_Y+1:Y_Block_Ind*Block_Size_Y);
        BlockMultiplication
    end
    %Process final Y_block (~blocksize)
    Y_Block = Y(:,Nr_Blocks_Y*Block_Size_Y+1:end);
    Y_Block_Ind = Y_Block_Ind+1;
    if(isempty(Y_Block_Ind))
        Y_Block_Ind = 1;
    end
    BlockMultiplication
end

%Process final X_block
X_Block = X(Nr_Blocks_X*Block_Size_X+1:end,:);
X_Block_Ind = X_Block_Ind+1;
if(isempty(X_Block_Ind))
    X_Block_Ind = 1;
end
%process Y_block with full blocksize
for Y_Block_Ind = 1:Nr_Blocks_Y
    Y_Block = Y(:,(Y_Block_Ind-1)*Block_Size_Y+1:Y_Block_Ind*Block_Size_Y);
    BlockMultiplication
end
%Process final Y_block (~blocksize)
Y_Block = Y(:,Nr_Blocks_Y*Block_Size_Y+1:end);
Y_Block_Ind = Y_Block_Ind+1;
if(isempty(Y_Block_Ind))
    Y_Block_Ind = 1;
end
BlockMultiplication 

%store final results
i = i(i~=0);j = j(i~=0);v = v(i~=0);
Z = sparse(i,j,v,Nr_Rows_X,Nr_Cols_Y);

catch
    %blocksizes are too large to fit in memory
    Block_Size_X = Block_Size_X/10;
    Block_Size_Y = Block_Size_Y/10;
    Z = Matrix_Multiplication_Blocks(X,Y,Block_Size_X,Block_Size_Y);
end
    
 
function BlockMultiplication
        %multiply both blocks and correct indices
        Z_Block = X_Block*Y_Block;
        [i_Block,j_Block,v_Block] = find(Z_Block);
        i_Block = i_Block + (X_Block_Ind-1)*Block_Size_X;
        j_Block = j_Block + (Y_Block_Ind-1)*Block_Size_Y;
        Nr_Elements_Block = length(i_Block);
        %store results in (i,j,v)
        try
            i(Pos_Current:Pos_Current+Nr_Elements_Block-1) = i_Block;
            j(Pos_Current:Pos_Current+Nr_Elements_Block-1) = j_Block;
            v(Pos_Current:Pos_Current+Nr_Elements_Block-1) = v_Block;
            Pos_Current = Pos_Current+Nr_Elements_Block;
        catch
            %i,j,v is currently too small
            i = [i;zeros(100*Nr_Elements_Block,1)];
            j = [j;zeros(100*Nr_Elements_Block,1)];
            v = [v;zeros(100*Nr_Elements_Block,1)];
            i(Pos_Current:Pos_Current+Nr_Elements_Block-1) = i_Block;
            j(Pos_Current:Pos_Current+Nr_Elements_Block-1) = j_Block;
            v(Pos_Current:Pos_Current+Nr_Elements_Block-1) = v_Block;
            Pos_Current = Pos_Current+Nr_Elements_Block;
        end    
end
        
end

