function [Alpha,Weak_Learners,R_AB,Time] = AdaCost(X,Y,C,T,R,mu)
%this function learns an AdaCost ensemble

%Input:
    %X,Y: training data
    %C: SVM parameter
    %T: nr of boosting rounds
    %R: cost ratio of C+/C-
    %mu: weight percentage

%initialize
Alpha = zeros(T,1);
Weak_Learners = cell(1,T);
Time = zeros(1,T);
R_AB = zeros(T,1);
m = size(X,1);
c = zeros(m,1);%cost of each instance
c(Y==1) = 1;% C+ equals 1
c(Y==-1) = 1/R;% C- = C+/R
D = c/sum(c);

%First boosting round
tic;
model = Train_WeakLearner(X,Y,D,C,mu,1);
h_X = Predict_WeakLearner(X,model);
r = sum(D.*Y.*h_X);
if (r > 1-1e-6 && r < 1+1e6)
    %r == 1
    Alpha(1) = 1;
    model.Ind_Pred = 'irregular';
    Weak_Learners{1} = model;
    R_AB(1) = 1;
    Time(1) = toc;
    return
elseif (r < 0)
    Alpha(1) = 1;
    Weak_Learners{1} = model;
    R_AB(1) = r;
    Time(1) = toc;
    return
else
    beta = Beta(Y,h_X,c);
    r_AC = sum(D.*Y.*h_X.*beta);
    alpha = log((1+r_AC)/(1-r_AC))/2;
    D = D.*exp(-alpha*Y.*h_X.*beta);
    D = D/sum(D);
    %store results 
    Alpha(1) = alpha;
    Weak_Learners{1} = model;
    R_AB(1) = r;
    Time(1) = toc; 
end

for t = 2:T
    tic;
    %train weak leaner under the current distribution D
    model = Train_WeakLearner(X,Y,D,C,mu,t);
    h_X = Predict_WeakLearner(X,model);%confidence rated predictions
    r = sum(D.*Y.*h_X);
    if ((r > 1-1e-6 && r < 1+1e6) || r < 0)
        Alpha(t) = 0;
        Alpha = Alpha/sum(Alpha);
        R_AB(t) = r;
        Time(t) = toc;
        return
    else
        %choose alpha
        beta = Beta(Y,h_X,c);
        r_AC = sum(D.*Y.*h_X.*beta);
        alpha = log((1+r_AC)/(1-r_AC))/2;
        %update
        D = D.*exp(-alpha*Y.*h_X.*beta);
        D = D/sum(D);
        %store results 
        Alpha(t) = alpha;
        Weak_Learners{t} = model;
        R_AB(t) = r;
        Time(t) = toc; 
    end
end
%update alpha
Alpha = Alpha/sum(Alpha);
end
