function [Alpha,Weak_Learners,R,Time] = EasyEnsemble(X,Y,C,T,S,mu)
%This function applies the EasyEnsemble algorithm.

%Input
    %X,Y: training data
    %C: C-parameter SVM used during boosting
    %T: number of boosting rounds
    %S: number of subsets
    %mu: weight percentage
%Output
    %Alpha: Matrix containing alpha's 
    %       (rows = subsets, cols = boosting rounds)
    %Weak_Learners: Matrix containing weak learners
    %       (rows = subsets, cols = boosting rounds)
    %R: Matrix containing R-values from adaboost
    %       (rows = subsets, cols = boosting rounds)
    %Time: Matrix containing timings

ind_maj = find(Y == -1);
ind_min = find(Y==1);
X_min = X(ind_min,:);
Y_min = Y(ind_min);
Alpha = zeros(S,T);
Weak_Learners = cell(S,T);
R = zeros(S,T);
Time = zeros(S,T);

for s = 1:S
    %randomly sample a majority class subset
    perm = randperm(length(ind_maj));
    ind_maj = ind_maj(perm);
    ind_maj_sub = ind_maj(1:length(ind_min));
    X_maj_sub = X(ind_maj_sub,:);
    Y_maj_sub = Y(ind_maj_sub);
    %apply Boosting algorithm
    X_train_s = [X_maj_sub;X_min];
    Y_train_s = [Y_maj_sub;Y_min];
    [Alpha_s,Weak_Learners_s,R_s,Time_s] = EasyEnsemble_Boosting(X_train_s,Y_train_s,C,T,mu);
    Alpha(s,:) = Alpha_s;
    Weak_Learners(s,:) = Weak_Learners_s;
    R(s,:) = R_s;
    Time(s,:) = Time_s;
end
end