function model = Train_WeakLearner(X,Y,D,C,mu,t)
%Train SVM model (using only p percent of tot. weight)

%Input
    %X,Y: initial training data (final train data will we adjusted according to p) 
    %D: weights given to X,Y by boosting process
    %C: C-value to train SVM (actual C will be adjusted)
    %mu: only use mu percent of total weight space to train model 
    %t: which round of boosting 
%Output
    %model: SVM model with extra field model.LR_weights
    
m = length(Y);
if(t == 1 && sum(D == 1/m) == m)
    %first boosting round and uniform distribution
    ind_min = find(Y==1); nr_min = length(find(Y==1));
    ind_maj = find(Y==-1);nr_maj = length(find(Y==-1));
    X_min = X(ind_min,:);X_maj = X(ind_maj,:);
    Y_min = Y(ind_min);Y_maj = Y(ind_maj);
    D_min = D(ind_min);D_maj = D(ind_maj);
    nr_min_train = floor((mu/100)*nr_min);
    nr_maj_train = floor((mu/100)*nr_maj);
    X_train_min = X_min(1:nr_min_train,:);Y_train_min = Y_min(1:nr_min_train);
    D_train_min = D_min(1:nr_min_train);
    X_train_maj = X_maj(1:nr_maj_train,:);Y_train_maj = Y_maj(1:nr_maj_train);
    D_train_maj = D_maj(1:nr_maj_train);
    X_train = [X_train_min;X_train_maj];
    Y_train = [Y_train_min;Y_train_maj];
    D_train = [D_train_min;D_train_maj];
    D_train = D_train/sum(D_train);%make a new distribution
    C_train = C/mean(D_train);

    train_opt = ['-q -s 3 -e 0.001 -B 1 -c ' num2str(C_train)];
    model = train(D_train,Y_train,X_train,train_opt);
    Y_train(Y_train == -1) = 0;
    [~,~,scores] = predict(Y_train,X_train,model,'-q');
    weights = glmfit(scores,Y_train,'binomial'); %fit LR model based on scores SVM
    model.LR_weights = weights;
    model.Ind_Pred = 'normal';
else
    %non-uniform distribution 
    [D_sort,pos] = sort(D,'descend');
    D_sort_sum = cumsum(D_sort);
    nr_train = find(D_sort_sum >= (mu/100-1e-06),1);
    ind_train = pos(1:nr_train);
    X_train = X(ind_train,:);
    Y_train = Y(ind_train);
    D_train = D(ind_train);
    if(isempty(find(Y_train == 1, 1)))
        %no pos intstances in train set -> add 1 pos instance
        disp('warning, no pos instances in training set; randomly added 1')
        ind_pos = find(Y == 1,1);
        X_train = [X_train;X(ind_pos,:)];Y_train = [Y_train;Y(ind_pos)];
        D_train = [D_train;D(pos(nr_train+1))];
    elseif(isempty(find(Y_train == -1, 1)))
        % no neg instances in train set -> add 1 neg instance
        disp('warning, no neg instances in training set; randomly added 1')
        ind_neg = find(Y == -1,1);
        X_train = [X_train;X(ind_neg,:)];Y_train = [Y_train;Y(ind_neg)];
        D_train = [D_train;D(pos(nr_train+1))];
    end
    D_train = D_train/sum(D_train);%make a new distribution
    C_train = C/mean(D_train);

    train_opt = ['-q -s 3 -e 0.001 -B 1 -c ' num2str(C_train)];
    model = train(D_train,Y_train,X_train,train_opt);
    Y_train(Y_train == -1) = 0;
    [~,~,scores] = predict(Y_train,X_train,model,'-q');
    weights = glmfit(scores,Y_train,'binomial'); %fit LR model based on scores SVM
    model.LR_weights = weights;
    model.Ind_Pred = 'normal';
end

