function [Alpha,Weak_Learners,R,Time] = EasyEnsemble_BeSim(X,Y,T,S)
%This function applies the EasyEnsemble algorithm with BeSim base learner

%Input
    %X,Y: training data
    %T: number of boosting rounds
    %S: number of subsets
%Output
    %Alpha: Matrix containing alpha's 
    %       (rows = subsets, cols = boosting rounds)
    %Weak_Learners: Matrix containing weak learners
    %       (rows = subsets, cols = boosting rounds)
    %R: Matrix containing R-values from adaboost
    %       (rows = subsets, cols = boosting rounds)
    %Time: Matrix containing timings

ind_maj = find(Y == -1);
Nr_min = length(find(Y==1));
Alpha = zeros(S,T);
Weak_Learners = cell(S,T);
R = zeros(S,T);
Time = zeros(S,T);

for s = 1:S
    %randomly sample a majority class subset
    perm = randperm(length(ind_maj));
    ind_maj = ind_maj(perm);
    ind_maj_sub = ind_maj(1:Nr_min);
    %apply Boosting algorithm
    [Alpha_s,Weak_Learners_s,R_s,Time_s] = EasyEnsemble_Boosting_BeSim(X,Y,ind_maj_sub,T);
    Alpha(s,:) = Alpha_s;
    Weak_Learners(s,:) = Weak_Learners_s;
    R(s,:) = R_s;
    Time(s,:) = Time_s;
end
end