function [Alpha,Weak_Learners,R,Time] = EasyEnsemble_Boosting_BeSim(X,Y,ind_maj_sub,T)
% Plain AdaBoost algorithm (with BeSim base learner)
%Input
    %(X,Y):original training data
    %ind_maj_sub: majority class instances that belong to subset under
    %             consideration
    %T: nr of boosting rounds 
    
%initialize
ind_min = find(Y==1);
ind = [ind_maj_sub;ind_min];
X_s = X(ind,:);
Y_s = Y(ind);
m = length(Y_s);
D = ones(m,1)/m;
Alpha = zeros(1,T);
R = zeros(1,T);
Weak_Learners = cell(1,T);%stores training indices from (X,Y)
Time = zeros(1,T);

%First boosting round
tic;

model = struct('Ind',ind);
h_X_s = Predict_WeakLearner_BeSim(X,Y,model,X_s);
r = sum(D.*Y_s.*h_X_s);
if ((r > 1-1e-6 && r < 1+1e6)|| r < 0)
    %r = 1 or r < 0
    Alpha(1) = 1;
    Weak_Learners{1} = model;
    R(1) = r;
    Time(1) = toc;
    return
else
    alpha = log((1+r)/(1-r))/2;
    D = D.*exp(-alpha*Y_s.*h_X_s);
    D = D/sum(D);
    %store results 
    Alpha(1) = alpha;
    Weak_Learners{1} = model;
    R(1) = r;
    Time(1) = toc; 
end

for t = 2:T
    tic;
    %train weak leaner under the current distribution D   
    model = Train_WeakLearner_BeSim(X,Y,ind,D);
    h_X_s = Predict_WeakLearner_BeSim(X,Y,model,X_s);
    r = sum(D.*Y_s.*h_X_s);
    if ((r > 1-1e-6 && r < 1+1e6) || r < 0)
        Alpha(t) = 0;
        Alpha = Alpha/sum(Alpha);
        R(t) = r;
        Time(t) = toc;
        return
    else
        %choose alpha 
        alpha = log((1+r)/(1-r))/2;
        %update
        D = D.*exp(-alpha*Y_s.*h_X_s);
        D = D/sum(D);
        %store results 
        Alpha(t) = alpha;
        Weak_Learners{t} = model;
        R(t) = r;
        Time(t) = toc; 
    end
end
%update alpha
Alpha = Alpha/sum(Alpha);
end