function [X_total,synth_samples] = OSR(X_min,X_maj,beta)
%This function applies a simple OverSampling with Replacement on the given
%minority instances. Each point of the X_min data is repeated by the same
%amount.

%nr of minority and majority examples
m_s = size(X_min,1);
m_l = size(X_maj,1);

%total amount of synth samples to be generated
G = ceil((m_l - m_s)*beta);

%find how many synth samples g_i are needed for each original sample
g_i = ceil(G/m_s);

%(repmat, fast enough since does no occur within loop here)
synth_samples = repmat(X_min,g_i,1);

%(repmat alternative implemenation -> tuns out to be slower than repmat)
% tic;
% [i,j,v] = find(X_min);
% L = length(i);
% I = zeros(L*g_i,1);
% i_start = 1;
% for iter = 1:g_i
%     I(i_start:i_start+L-1) = i+(iter-1)*m_s;
%     i_start = i_start+L;
% end
% J = repmat(j,g_i,1);
% V = repmat(v,g_i,1);
% synth_samples_2 = sparse(I,J,V,g_i*m_s,size(X_min,2));
% telap = toc;
% disp(['time for alternative repmat in sec: ',num2str(telap)]);


%Randomly remove synth points until total nr of synth samples equals G
while(size(synth_samples,1) ~= G)
    pos = randi(size(synth_samples,1),size(synth_samples,1)-G,1);
    synth_samples = removerows(synth_samples,'ind',pos);
end

X_total = [X_min;synth_samples];



end

