function [nn,sim] = nearest_neighbours( X,Y,k,sim_measure,blocksize_x,blocksize_y)

if(~exist('blocksize_x','var'))
    blocksize_x = 1000;
end

if(~exist('blocksize_y','var'))
    blocksize_y = 100000;
end

nr_points_X = size(X,1);
nr_points_Y = size(Y,1);

nn = zeros(nr_points_X,k);
sim = zeros(nr_points_X,k);

k_blocks = min(k,blocksize_y);
nr_nns_X_block = k_blocks*floor(nr_points_Y/blocksize_y)+ min(k,mod(nr_points_Y,blocksize_y));

%Process the X_blocks with full blocksize
nr_blocks_X = floor(nr_points_X/blocksize_x);
x_start = 1;%init
nn_X_block = zeros(blocksize_x,nr_nns_X_block);%init
sim_X_block = zeros(blocksize_x,nr_nns_X_block);%init
for block_count_x = 1:nr_blocks_X
    %X block that is processed in this loop
    X_block = X(x_start:x_start+blocksize_x-1,:);
    %Process the Y_blocks with full blocksize 
    nr_blocks_Y = floor(nr_points_Y/blocksize_y);
    y_start = 1;
    for block_count_y = 1:nr_blocks_Y
        %Y block that is processed in this loop
        Y_block = Y(y_start:y_start+blocksize_y-1,:);
        %calculate nn & sim for full {X_block,Y_block} combination
        [nn_XY_block,sim_XY_block] = nearest_neighbours_temp(X_block,Y_block,k_blocks,sim_measure);
        nn_X_block(:,(block_count_y-1)*k_blocks+1:block_count_y*k_blocks)= nn_XY_block + (block_count_y-1)*blocksize_y;
        sim_X_block(:,(block_count_y-1)*k_blocks+1:block_count_y*k_blocks)= sim_XY_block;
        %update
        y_start = y_start + blocksize_y;
    end
    %process final Y_block (size <= blocksize)
    Y_block = Y(y_start:end,:);
    if(size(Y_block,1) > 0)
        k_used = min(k,size(Y_block,1));
        [nn_XY_block,sim_XY_block] = nearest_neighbours_temp(X_block,Y_block,k_used,sim_measure);
        nn_X_block(:,nr_blocks_Y*k_blocks+1:end)= nn_XY_block + nr_blocks_Y*blocksize_y;
        sim_X_block(:,nr_blocks_Y*k_blocks+1:end)= sim_XY_block;
    end
    %determine nn and sim for this X_block
    [sim_X_block_sort,ix2] = sort(sim_X_block,2,'descend');%ix2 gives col indices
    ix2 = ix2(:,1:k);%we only need the first k columns
    ix1 = repmat([1:blocksize_x]',[1 k]);%ix1 gives row indices
    ix = sub2ind([blocksize_x nr_nns_X_block],ix1,ix2);%ix gives linear indices
    nn_X_block_sort = nn_X_block(ix);
    
    sim(x_start:x_start+blocksize_x-1,:) = sim_X_block_sort(:,1:k);
    nn(x_start:x_start+blocksize_x-1,:) = nn_X_block_sort;
    %update for next X_block
    x_start = x_start + blocksize_x;    
end

%Process final X_block (size <= blocksize)
X_block = X(x_start:end,:);
nr_X_block = size(X_block,1);

if(nr_X_block > 0)
    %init
    nn_X_block = zeros(nr_X_block,nr_nns_X_block);
    sim_X_block = zeros(nr_X_block,nr_nns_X_block); 
    %Process the Y_blocks with full blocksize 
    nr_blocks_Y = floor(nr_points_Y/blocksize_y);
    y_start = 1;
    for block_count_y = 1:nr_blocks_Y
        %Y block that is processed in this loop
        Y_block = Y(y_start:y_start+blocksize_y-1,:);
        %calculate nn & sim for full {X_block,Y_block} combination
        [nn_XY_block,sim_XY_block] = nearest_neighbours_temp(X_block,Y_block,k_blocks,sim_measure);
        nn_X_block(:,(block_count_y-1)*k_blocks+1:block_count_y*k_blocks)= nn_XY_block + (block_count_y-1)*blocksize_y;
        sim_X_block(:,(block_count_y-1)*k_blocks+1:block_count_y*k_blocks)= sim_XY_block;
        %update
        y_start = y_start + blocksize_y;
    end  
    %process final Y_block (size <= blocksize)
    Y_block = Y(y_start:end,:);
    if(size(Y_block,1) > 0)
        k_used = min(k,size(Y_block,1));
        [nn_XY_block,sim_XY_block] = nearest_neighbours_temp(X_block,Y_block,k_used,sim_measure);
        nn_X_block(:,nr_blocks_Y*k_blocks+1:end)= nn_XY_block + nr_blocks_Y*blocksize_y;
        sim_X_block(:,nr_blocks_Y*k_blocks+1:end)= sim_XY_block;
    end
    %determine nn and sim for this X_block
    [sim_X_block_sort,ix2] = sort(sim_X_block,2,'descend');%ix2 gives col indices
    ix2 = ix2(:,1:k);%we only need the first k columns
    ix1 = repmat([1:nr_X_block]',[1 k]);%ix1 gives row indices
    ix = sub2ind([nr_X_block nr_nns_X_block],ix1,ix2);%ix gives linear indices
    nn_X_block_sort = nn_X_block(ix);
    
    sim(x_start:end,:) = sim_X_block_sort(:,1:k);
    nn(x_start:end,:) = nn_X_block_sort;
end