function [ X_train,Y_train ] = Closest_Knn(X_min,X_maj,beta,sim_measure,K,sim)
%The majority class instances are kept that are closest to the minority
%class instances.
%(keep the majority instances whose similarity with the K closest
% minority instances is the largest)

%param:
%   X_min: minority training instances
%   X_maj: majority training instances
%   beta: controls how many majority instances are discarded
%   sim_measure
%   K: how many minority nearest neighbours are used

%   sim (optional): similarity computation for this K-value & sim_measure 
%                   sim = sim_knn_total(X_maj,X_min,K,sim_measure)


m_s = size(X_min,1);
m_l = size(X_maj,1);

%number of maj instances to be removed
Nr_Remove = floor(beta*(m_l-m_s));
if(beta ==0)
    X_train = [X_min;X_maj];
    Y_train = [ones(m_s,1);-1*ones(m_l,1)];
    return
end

%Calculate total similarity with the k-nn of X_min
if(~exist('sim','var') || isempty(sim))
    sim = sim_knn_total(X_maj,X_min,K,sim_measure,1000,100000);
end
%determine the majority instances to be removed
[~,Ind] = sort(sim,'ascend');
Ind_Remove = Ind(1:Nr_Remove);

%downsample X_maj
X_maj_down = removerows(X_maj,'ind',Ind_Remove);

%construct new training data
X_train = [X_min;X_maj_down];
Y_train = [ones(m_s,1);-1*ones(size(X_maj_down,1),1)];


end

