function [X_train,Y_train,C_maj,telap_iter_genlouv] = Cluster_Based_US_Direct(X_min,X_maj,beta,option,C_maj)
%CLUSTER_BASED_US, working on bigraph directly
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Make sure to add bigraph scripts and genlouvain community detection
%toolbox to your path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m_s = size(X_min,1);
m_l = size(X_maj,1);

%Cluster the majority class
if(~exist('C_maj','var') || isempty(C_maj))
    B = bipartite_f(X_maj);%obtain modularity matrix
    tstart_iter_genlouv = tic;
    [C_maj,~,~] = iterated_genlouvain(B,10000,0);%C_maj also contains cluster results for the top nodes (features)
    telap_iter_genlouv = toc(tstart_iter_genlouv);
    C_maj = C_maj(1:m_l);
end
%obtain partition matrix and cluster sizes
Nr_Clust = length(unique(C_maj));
rows = 1:1:m_l;
cols = C_maj;
values = ones(size(rows));
P_maj = sparse(rows,cols,values,m_l,Nr_Clust);

%Select majority class instances
Nr_Remove = floor(beta*(m_l-m_s));
Nr_Keep = m_l - Nr_Remove;
if(Nr_Keep < Nr_Clust)
    X_maj_chosen_ind = zeros(Nr_Keep,1);
    %sort clusters (according to option param) and randomly select 1 instance from
    %the first Nr_Keep clusters
    Clust_Size = full(sum(P_maj,1));
    if(strcmp(option,'C_Smallest'))
        [~,ind] = sort(Clust_Size,'ascend');
    elseif(strcmp(option,'C_Largest'))
        [~,ind] = sort(Clust_Size,'descend');
    else
        disp('wrong choice of option parameter')
    end
    Clust_Chosen = ind(1:Nr_Keep);
    for i = 1:Nr_Keep
        Clust_Current = Clust_Chosen(i);
        X_maj_ind = find(P_maj(:,Clust_Current));
        pos = randi(length(X_maj_ind));
        X_maj_chosen_ind(i) = X_maj_ind(pos);
    end
else
    Nr_Inst_Per_C = ceil(Nr_Keep/Nr_Clust);
    X_maj_chosen_ind = zeros(Nr_Clust*Nr_Inst_Per_C,1);
    for c = 1:Nr_Clust
        X_maj_ind = find(P_maj(:,c));
        pos = randi(length(X_maj_ind),Nr_Inst_Per_C,1);
        X_maj_chosen_ind(Nr_Inst_Per_C*(c-1)+1:c*Nr_Inst_Per_C) = X_maj_ind(pos);
    end
    perm = randperm(Nr_Clust*Nr_Inst_Per_C);
    X_maj_chosen_ind = X_maj_chosen_ind(perm(1:Nr_Keep));    
end
X_maj_down = X_maj(X_maj_chosen_ind,:);
X_train = [X_min;X_maj_down];
Y_train = [ones(m_s,1);-1*ones(size(X_maj_down,1),1)];

end




