function sim = sim_knn_total( X,Y,k,sim_measure,blocksize_x,blocksize_y)
%for each x in X, this function computes the TOTAL similarity with 
% the k nearest neighbours in Y

if(~exist('blocksize_x','var'))
    blocksize_x = 1000;
end

if(~exist('blocksize_y','var'))
    blocksize_y = 100000;
end
 
nr_points_X = size(X,1);
nr_points_Y = size(Y,1);

sim = zeros(nr_points_X,1);

k_blocks = min(k,blocksize_y);
nr_nns_X_block = k_blocks*floor(nr_points_Y/blocksize_y)+ min(k,mod(nr_points_Y,blocksize_y));

%Process the X_blocks with full blocksize
nr_blocks_X = floor(nr_points_X/blocksize_x);
x_start = 1;%init
i_X_block = [];j_X_block = [];v_X_block = [];%sim_X_block = sparse(i_X_block ;j_X_block ;v_X_block)

for block_count_x = 1:nr_blocks_X
    %X block that is processed in this loop
    X_block = X(x_start:x_start+blocksize_x-1,:);
    %Process the Y_blocks with full blocksize 
    nr_blocks_Y = floor(nr_points_Y/blocksize_y);
    y_start = 1;
    for block_count_y = 1:nr_blocks_Y
        %Y block that is processed in this loop
        Y_block = Y(y_start:y_start+blocksize_y-1,:);
        %calculate sim for full {X_block,Y_block} combination
        sim_XY_block = sim_knn_basic(X_block,Y_block,k_blocks,sim_measure);
        [i,j,v] = find(sim_XY_block);
        i_X_block = [i_X_block;i(:)];
        j_X_block = [j_X_block;j(:)+(block_count_y-1)*k_blocks];
        v_X_block = [v_X_block;v(:)];
        %update
        y_start = y_start + blocksize_y;
    end
    %process final Y_block (size <= blocksize)
    Y_block = Y(y_start:end,:);
    if(size(Y_block,1) > 0)
        k_used = min(k,size(Y_block,1));
        sim_XY_block = sim_knn_basic(X_block,Y_block,k_used,sim_measure);
        [i,j,v] = find(sim_XY_block);
        i_X_block = [i_X_block;i(:)];
        j_X_block = [j_X_block;j(:)+nr_blocks_Y*k_blocks];
        v_X_block = [v_X_block;v(:)];        
    end
    %sort sim_X_block
    sim_X_block = sparse(i_X_block,j_X_block,v_X_block,blocksize_x,nr_nns_X_block);
    sim_X_block_sort = sort(sim_X_block,2,'descend');
    sim_X_block_sort = sim_X_block_sort(:,1:k);%we only need the first k columns
    %store TOTAL similarity of this X_block (with it's k nn)
    sim(x_start:x_start+blocksize_x-1,:) = sum(sim_X_block_sort,2);
    %update for next X_block
    i_X_block = [];j_X_block = [];v_X_block = [];
    x_start = x_start + blocksize_x;    
end

%Process final X_block (size <= blocksize)
X_block = X(x_start:end,:);
nr_X_block = size(X_block,1);
if(nr_X_block > 0)
    %init
    i_X_block = [];j_X_block = [];v_X_block = [];%sim_X_block = sparse(i_X_block ;j_X_block ;v_X_block)
    %Process the Y_blocks with full blocksize 
    nr_blocks_Y = floor(nr_points_Y/blocksize_y);
    y_start = 1;
    for block_count_y = 1:nr_blocks_Y
        %Y block that is processed in this loop
        Y_block = Y(y_start:y_start+blocksize_y-1,:);
        %calculate sim for full {X_block,Y_block} combination
        sim_XY_block = sim_knn_basic(X_block,Y_block,k_blocks,sim_measure);
        [i,j,v] = find(sim_XY_block);
        i_X_block = [i_X_block;i(:)];
        j_X_block = [j_X_block;j(:)+(block_count_y-1)*k_blocks];
        v_X_block = [v_X_block;v(:)];
        %update
        y_start = y_start + blocksize_y;
    end  
    %process final Y_block (size <= blocksize)
    Y_block = Y(y_start:end,:);
    if(size(Y_block,1) > 0)
        k_used = min(k,size(Y_block,1));
        sim_XY_block = sim_knn_basic(X_block,Y_block,k_used,sim_measure);
        [i,j,v] = find(sim_XY_block);
        i_X_block = [i_X_block;i(:)];
        j_X_block = [j_X_block;j(:)+nr_blocks_Y*k_blocks];
        v_X_block = [v_X_block;v(:)];  
    end
    %sort sim_X_block
    sim_X_block = sparse(i_X_block,j_X_block,v_X_block,nr_X_block,nr_nns_X_block);
    sim_X_block_sort = sort(sim_X_block,2,'descend');
    sim_X_block_sort = sim_X_block_sort(:,1:k);%we only need the first k columns
    %store TOTAL similarity of this X_block (with it's k nn)
    sim(x_start:end,:) = sum(sim_X_block_sort,2);
end
