function sim = total_sim( X,Y,sim_measure,blocksize_x,blocksize_y)
%Does the same as the 'basic' version, however the computation is
%decomposed into several blocks

if(~exist('blocksize_x','var'))
    blocksize_x = 100000;
end

if(~exist('blocksize_y','var'))
    blocksize_y = 1000;
end

nr_points_X = size(X,1);
nr_points_Y = size(Y,1);
sim = zeros(nr_points_X,1);
nr_blocks_X = floor(nr_points_X/blocksize_x);
nr_blocks_Y = floor(nr_points_Y/blocksize_y);

%Process the X_blocks with full blocksize
x_start = 1;%init
for block_count_x = 1:nr_blocks_X
    %X block that is processed in this loop
    X_block = X(x_start:x_start+blocksize_x-1,:);
    %Process the Y_blocks with full blocksize 
    y_start = 1;
    for block_count_y = 1:nr_blocks_Y
        %Y block that is processed in this loop
        Y_block = Y(y_start:y_start+blocksize_y-1,:);
        %calculate total sim for {X_block,Y_block} combination
        sim_XY_block = total_sim_basic(X_block,Y_block,sim_measure);
        sim(x_start:x_start+blocksize_x-1) = sim(x_start:x_start+blocksize_x-1)+ sim_XY_block;
        %update
        y_start = y_start + blocksize_y;
    end
    %process final Y_block (size <= blocksize)
    Y_block = Y(y_start:end,:);
    if(size(Y_block,1) > 0)
        %calculate total sim for {X_block,Y_block} combination
        sim_XY_block = total_sim_basic(X_block,Y_block,sim_measure);
        sim(x_start:x_start+blocksize_x-1) = sim(x_start:x_start+blocksize_x-1)+ sim_XY_block;
    end
    %update for next X_block
    x_start = x_start + blocksize_x;    
end

%Process final X_block (size <= blocksize)
X_block = X(x_start:end,:);
if(size(X_block,1) > 0)
    %Process the Y_blocks with full blocksize 
    y_start = 1;
    for block_count_y = 1:nr_blocks_Y
        %Y block that is processed in this loop
        Y_block = Y(y_start:y_start+blocksize_y-1,:);
        %calculate total sim for {X_block,Y_block} combination
        sim_XY_block = total_sim_basic(X_block,Y_block,sim_measure);
        sim(x_start:end) = sim(x_start:end)+ sim_XY_block;
        %update
        y_start = y_start + blocksize_y;
    end
    %process final Y_block (size <= blocksize)
    Y_block = Y(y_start:end,:);
    if(size(Y_block,1) > 0)
        %calculate total sim for {X_block,Y_block} combination
        sim_XY_block = total_sim_basic(X_block,Y_block,sim_measure);
        sim(x_start:end) = sim(x_start:end)+ sim_XY_block;
    end
end

end

